@@ -48,6 +48,8 @@ class Api(snappi.Api):
         self._config = None
         self._assistant = None
         self._ixn_errors = list()
+        self._config_objects = {}
+        self._device_encap = {}
         self.validation = Validation(self)
         self.vport = Vport(self)
         self.ngpf = Ngpf(self)
@@ -74,6 +76,9 @@ class Api(snappi.Api):
     def get_config_object(self, name):
         return self._config_objects[name]
 
+    def get_device_encap(self, name):
+        return self._device_encap[name]
+
     @property
     def ixn_objects(self):
         """A dict of all model unique names to ixn hrefs
@@ -107,25 +112,26 @@ class Api(snappi.Api):
             o.__dict__[k] = self._dict_to_obj(v)
         return o
 
-    # def _request_detail(self):
-    #     request_detail = RequestDetail()
-    #     errors = self._errors
-    #     warnings = list()
-    #     app_errors = self._globals.AppErrors.find()
-    #     if len(app_errors) > 0:
-    #         current_errors = app_errors[0].Error.find()
-    #         if len(current_errors) > 0:
-    #             for error in current_errors:
-    #                 match = [o for o in self._ixn_errors if o.Name == error.Name
-    #                                                 and o.LastModified == error.LastModified]
-    #                 if len(match) == 0:
-    #                     if error.ErrorLevel == 'kWarning':
-    #                         warnings.append("IxNet - {0}".format(error.Description))
-    #                     if error.ErrorLevel == 'kError':
-    #                         errors.append("IxNet - {0}".format(error.Description))
-    #     request_detail.errors = errors
-    #     request_detail.warnings = warnings
-    #     return request_detail
+    def _request_detail(self):
+        request_detail = snappi.Details()
+        errors = self._errors
+        warnings = list()
+        app_errors = self._globals.AppErrors.find()
+        if len(app_errors) > 0:
+            current_errors = app_errors[0].Error.find()
+            if len(current_errors) > 0:
+                for error in current_errors:
+                    # Add loop as sequence are not sorted
+                    match = [o for o in self._ixn_errors if o.Name == error.Name
+                                                    and o.LastModified == error.LastModified]
+                    if len(match) == 0:
+                        if error.ErrorLevel == 'kWarning':
+                            warnings.append("IxNet - {0}".format(error.Description))
+                        if error.ErrorLevel == 'kError':
+                            errors.append("IxNet - {0}".format(error.Description))
+        request_detail.errors = errors
+        request_detail.warnings = warnings
+        return request_detail
     
     def set_config(self, config):
         """Set or update the configuration
@@ -134,9 +140,8 @@ class Api(snappi.Api):
             config = self.config().deserialize(config)
         self._config = config
         self._config_objects = {}
+        self._device_encap = {}
         self._ixn_objects = {}
-        self._capture_request = None
-        self._errors = []
         self._connect()
         with Timer(self, 'Config validation'):
             self.validation.validate_config()
@@ -149,6 +154,7 @@ class Api(snappi.Api):
             with Timer(self, 'Flows configuration'):
                 self.traffic_item.config()
         self._running_config = self._config
+        return self._request_detail()
 
     def set_transmit_state(self, flow_transmit_state):
         """Set the transmit state of flows
@@ -157,125 +163,71 @@ class Api(snappi.Api):
             flow_transmit_state = self.transmit_state().deserialize(
                                     flow_transmit_state)
         self._connect()
-        return self.traffic_item.transmit(flow_transmit_state)
+        self.traffic_item.transmit(flow_transmit_state)
+        return self._request_detail()
 
     def set_link_state(self, link_state):
+        self._connect()
         if link_state.port_names is not None:
             self.vport.set_link_state(link_state)
+        return self._request_detail()
     
     def set_capture_state(self, request):
         """Starts capture on all ports that have capture enabled.
         """
         self._connect()
-        self._capture_request = request
-        if self._capture_request is not None and request.state == 'stop':
-            self._stop_capture()
-
-    def _start_capture(self):
-        if self._capture_request is not None:
-            with Timer(self, 'Captures start'):
-                payload = {'arg1': []}
-                for vport in self.select_vports().values():
-                    payload['arg1'].append(vport['href'])
-                url = '%s/vport/operations/clearCaptureInfos' % self._ixnetwork.href
-                self._request('POST', url, payload)
-                self._ixnetwork.StartCapture()
-
-    def _stop_capture(self):
-        with Timer(self, 'Captures stop'):
-            if self._capture_request.port_names:
-                payload = {'arg1': []}
-                for vport_name, vport in self.select_vports().items():
-                    if vport_name in self._capture_request.port_names:
-                        payload['arg1'].append(vport['href'])
-                url = '%s/vport/operations/clearCaptureInfos' % self._ixnetwork.href
-                self._request('POST', url, payload)
-            else:
-                self._ixnetwork.StopCapture()
+        self.capture.set_capture_state(request)
+        return self._request_detail()
 
     def get_capture(self, request):
         """Gets capture file and returns it as a byte stream
         """
-        with Timer(self, 'Captures stop'):
-            capture = self._vport.find(Name=request.port_name).Capture
-            capture.Stop('allTraffic')
-
-            #   Internally setting max time_out to 90sec with 3sec polling interval.
-            #   Todo: Need to discuss and incorporate time_out field within model
-            retry_count = 30
-            port_ready = True
-            for x in range(retry_count):
-                port_ready = True
-                time.sleep(3)
-                capture = self._vport.find(Name=request.port_name).Capture
-                if capture.HardwareEnabled and capture.DataCaptureState == 'notReady':
-                    port_ready = False
-                    continue
-                if capture.SoftwareEnabled and capture.ControlCaptureState == 'notReady':
-                    port_ready = False
-                    continue
-                break
-            if not port_ready:
-                self.warning("Capture was not stopped for this port %s" %
-                             (request.port_name))
-
-        payload = {'arg1': [self._vport.href]}
-        url = '%s/vport/operations/getCaptureInfos' % self._ixnetwork.href
-        response = self._request('POST', url, payload)
-        file_name = response['result'][0]['arg6']
-        file_id = response['result'][0]['arg1']
-
-        url = '%s/vport/operations/saveCaptureInfo' % self._ixnetwork.href
-        payload = {'arg1': self._vport.href, 'arg2': file_id}
-        response = self._request('POST', url, payload)
-
-        url = '%s/vport/operations/releaseCapturePorts' % self._ixnetwork.href
-        payload = {'arg1': [self._vport.href]}
-        response = self._request('POST', url, payload)
-
-        path = '%s/capture' % self._ixnetwork.Globals.PersistencePath
-        url = '%s/files?absolute=%s&filename=%s.cap' % (self._ixnetwork.href,
-                                                        path, file_name)
-        pcap_file_bytes = self._request('GET', url)
-        return pcap_file_bytes
-
-    def get_port_metrics(self, request):
-        """Abstract API implementation
-        """
-        self._errors = []
-        if isinstance(request, (type(self.port_metrics_request()),
+        self._connect()
+        if isinstance(request, (type(self.capture_request()),
                                 str)) is False:
             raise TypeError(
-                'The content must be of type Union[PortMetricsRequest, str]')
+                'The content must be of type Union[CaptureRequest, str]')
         if isinstance(request, str) is True:
-            request = self.port_metrics_request().deserialize(
+            request = self.capture_request().deserialize(
                 request)
-        response = self.vport.results(request)
-        if len(self._errors) > 0:
-            raise Exception('\n'.join(self._errors))
-        return self.port_metrics().deserialize(response)
-
-    def get_flow_metrics(self, request):
-        """Abstract API implementation
+        return self.capture.results(request)
+    
+    def get_metrics(self, request):
+        """
+        Gets port, flow and protocol metrics.
 
         Args
         ----
-        - request (Union[FlowRequest, str]): A request for flow results.
-            The request content MUST be based on the OpenAPI #/components/schemas/Result.FlowRequest model.
-            See the docs/openapi.yaml document for all model details.
+        - request (Union[MetricsRequest, str]): A request for Port, Flow and
+          protocol metrics.
+          The request content MUST be vase on the OpenAPI model,
+          #/components/schemas/Result.MetricsRequest
+          See the docs/openapi.yaml document for all model details
         """
-        self._errors = []
-        if isinstance(request, (type(self.flow_metrics_request()),
+        self._connect()
+        metric_req = self.metrics_request()
+        if isinstance(request, (type(metric_req),
                                 str)) is False:
             raise TypeError(
-                'The content must be of type Union[FlowMetricsRequest, str]')
+                'The content must be of type Union[MetricsRequest, str]')
         if isinstance(request, str) is True:
-            request = self.flow_metrics_request().deserialize(
-                request)
-        response = self.traffic_item.results(request)
-        if len(self._errors) > 0:
-            raise Exception('\n'.join(self._errors))
-        return self.flow_metrics().deserialize(response)
+            request = metric_req.deserialize(request)
+        # Need to change the code style when the choice Enum grows big
+        if request.choice == 'port':
+            response = self.vport.results(request.port)
+            metric_res = self.metrics_response()
+            metric_res.port_metrics.deserialize(response)
+            return metric_res
+        if request.choice == 'flow':
+            response = self.traffic_item.results(request.flow)
+            metric_res = self.metrics_response()
+            metric_res.flow_metrics.deserialize(response)
+            return metric_res
+        if request.choice is not None:
+            msg = "{} is not a supported choice for metrics; \
+            the supported choices are \
+            ['port', 'flow']".format(request.choice)
+            raise Exception(msg)
 
     def add_error(self, error):
         """Add an error to the global errors
@@ -288,6 +240,7 @@ class Api(snappi.Api):
     def _connect(self):
         """Connect to an IxNetwork API Server.
         """
+        self._errors = []
         if self._assistant is None:
             platform = TestPlatform(self._address, rest_port=self._port)
             platform.Authenticate(self._username, self._password)
