@@ -239,12 +239,8 @@ class TrafficItem(CustomField):
                 if device.__class__ \
                         is snappi.DeviceBgpv4RouteRange:
                     encap = 'ipv4'
-                elif device.choice is 'ethernet':
-                    encap = 'ethernetVlan'
-                elif device.choice is 'bgpv4':
-                    encap = 'ipv4'
                 else:
-                    encap = device.choice
+                    encap = self._api.get_device_encap(name)
         return encap
 
     def _configure_endpoint(self, ixn_endpoint_set, endpoint):
@@ -551,7 +547,7 @@ class TrafficItem(CustomField):
                 with Timer(self._api, 'Flows clear statistics'):
                     self._api._ixnetwork.ClearStats(
                         ['waitForPortStatsRefresh', 'waitForTrafficStatsRefresh'])
-            self._api._start_capture()
+            self._api.capture._start_capture()
         self._api._traffic_item.find(Name=regex)
         if len(self._api._traffic_item) > 0:
             if request.state == 'start':
@@ -614,19 +610,24 @@ class TrafficItem(CustomField):
         """Return flow results
         """
         # setup parameters
-        if request.column_names is None:
+        self._column_names = request._properties.get('column_names')
+        if self._column_names is None:
             self._column_names = []
-        else:
-            self._column_names = request.column_names
+        elif not isinstance(self._column_names, list):
+            msg = "Invalid format of column_names passed {},\
+                    expected list".format(self._column_names)
+            raise Exception(msg)
+
+        flow_names = request._properties.get('flow_names')
+        if flow_names is None or len(flow_names) == 0:
+            flow_names = [flow.name for flow in self._api._config.flows]
+        elif not isinstance(flow_names, list):
+            msg = "Invalid format of flow_names passed {},\
+                    expected list".format(flow_names)
+            raise Exception(msg)
+
         filter = {'property': 'name', 'regex': '.*'}
-        flow_names = [flow.name for flow in self._api._config.flows]
-        if request is not None and request.flow_names is not None and len(
-                request.flow_names) > 0:
-            flow_names = request.flow_names
-        if len(flow_names) == 1:
-            filter['regex'] = '^%s$' % flow_names[0]
-        elif len(flow_names) > 1:
-            filter['regex'] = '^(%s)$' % '|'.join(flow_names)
+        filter['regex'] = '^(%s)$' % '|'.join(flow_names)
 
         # initialize result values
         flow_rows = {}
